function RequestsFactory(domain) {
  this.domain = domain || '';
  this.format = '.smarttvjson';
  this.contentUrls = {};

  this.contentUrls[ContentTypes.CHANNELS] = 'live';
  this.contentUrls[ContentTypes.QUEUE] = 'queue';

  this.requests = {};
}

// public
RequestsFactory.prototype.abort = function(requestId) {
  for (var rId in this.requests) {
    if (!requestId || requestId == rId) {
      // samsung 2012. It calls xhr.success() with 'success' status immediately after call xhr.abort()
      var r = this.requests[rId];
      delete this.requests[rId];
      r && r.abort();
    }
  }
}

//////////////// AUTH /////////////////////////
// public
RequestsFactory.prototype.getToken = function(deviceId, deviceType, context, callback) {
  var self = this;
  this.makeRequest({
    data: 'device_id=' + deviceId + '&device_type=' + deviceType,
    url: this.domain + 'devices/rendezvous/request' + this.format,
  }, [{scope: context, fn: callback}], 'getToken');
}

RequestsFactory.prototype.checkTokenConfirmed = function(deviceId, deviceType, code, context, callback) {
  var self = this;
  this.makeRequest({
    data: 'device_id=' + deviceId + '&device_type=' + deviceType + '&code=' + code,
    url: this.domain + 'devices/rendezvous/result' + this.format,
  }, [{scope: context, fn: callback}], 'confirm');
}

RequestsFactory.prototype.verifyToken = function(context, callback) {
  var self = this;
  this.makeRequest({
    url: this.domain + 'session/verify' + this.format,
  }, [{scope: context, fn: callback}], 'verify');
}


///////////////// LOG ////////////////////////////
//public
RequestsFactory.prototype.logInfo = function(userId, logData) {
  this.makeRequest({
    url: this.domain + 'samsung_logs',
    type: 'POST',
    data: {user_id: userId, log: logData}
  }, null, 'logInfo');
}

////////////// ACCOUNT ///////////////////////////

// public
RequestsFactory.prototype.getAccount = function(callbacks, options, backId) {
  this.makeRequest({
    url: this.domain + 'account' + this.format
  }, callbacks, backId);
}

//////////// VERSION ////////////////////////////

// public
RequestsFactory.prototype.getLastVersion = function(domain, callbacks, options, backId) {
  this.makeRequest({
    url: domain + 'Widget/last_version.json' + '?' + (new Date()).valueOf()
  }, callbacks, backId);
}

//////////// CONTENT ////////////////////////////

// public
RequestsFactory.prototype.getQueue = function(page, callbacks, options, backId) {
  return this.makeRequest({
    url: this.domain + this.contentUrls[ContentTypes.QUEUE] + this.format,
    data: {page: page},
    async: !(options || {}).sync
  }, callbacks, backId);
}

// public
RequestsFactory.prototype.getChannels = function(callbacks, options, backId) {
  this.makeRequest({
    url: this.domain + this.contentUrls[ContentTypes.CHANNELS] + this.format
  }, callbacks, backId);
}

// public
RequestsFactory.prototype.getChannelTelecasts = function(channelAbbr, callbacks, options, backId) {
  this.makeRequest({
    url: this.domain + this.contentUrls[ContentTypes.CHANNELS] + '/' + channelAbbr + this.format,
  }, callbacks, backId);
}

/////////////// ACTIONS WITH CONTENT ////////////////////////

// public
RequestsFactory.prototype.removeChannelFromFav = function(abbr, callbacks, options, backId) {
  this.makeRequest({
    url: this.domain + 'account/channel_favourites/' + abbr + this.format,
    type: 'DELETE',
  }, callbacks, backId);
}

// public
RequestsFactory.prototype.addChannelToFav = function(abbr, callbacks, options, backId) {
  alert("RequestsFactory.addChannelToFav " + abbr);
  this.makeRequest({
    url: this.domain + 'account/channel_favourites/' + abbr + this.format,
    type: 'PUT',
    data: abbr
  }, callbacks, backId);
}

// public
RequestsFactory.prototype.removeFilmFromQueue = function(filmQueueId, callbacks, options, backId) {
  this.makeRequest({
    url: this.domain + 'queue/' + filmQueueId + this.format,
    type: 'DELETE'
  }, callbacks, backId);
}

///////////////////////////////////////////////////////////////////////////////////////////////////

// public
RequestsFactory.prototype.saveHistoty = function(airtimeId, playheadTime) {
  this.makeRequest({
    url: this.domain + '/history.js',
    type: 'POST',
    data: {
      airtime_id: airtimeId,
      playhead_time: playheadTime
    }
  }, 'saveHistory');
}

//////////////////////////////////////////////////////////////////////////////////////////////////

// private
RequestsFactory.prototype.makeRequest = function(options, callbacks, id) {
  var self = this;
  var cbs = callbacks || [];
  var r = {
    dataType: 'json',
    success: function(data, textStatus, jqXHR) {
      if (!self.requests[id])
        return;
      for (var i = 0; i < cbs.length; i++) {
        var c = cbs[i];
        if (c.sfn) {
          c.sfn.call(c.scope, id, data);
        } else if (c.fn) {
          c.fn.call(c.scope, true, data);
        }
      }
      self.completeRequest(id);
    },
    error: function(jqXHR, textStatus, errorThrown) {
      if (!self.requests[id])
        return;
      for (var i = 0; i < cbs.length; i++) {
        var c = cbs[i];
        if (c.ffn) {
          c.ffn.call(c.scope, id, self.getRequestError(jqXHR, textStatus, errorThrown));
        } else if (c.fn) {
          c.fn.call(c.scope, false, self.getRequestError(jqXHR, textStatus, errorThrown));
        }
      }
      self.completeRequest(id);
    },
    beforeSend: self.setToken
  }
  var xhr = $.ajax($.extend(r, options));
  this.requests[id] = xhr;
}

// private
RequestsFactory.prototype.completeRequest = function(id) {
  delete this.requests[id];
}

// private
RequestsFactory.prototype.setToken = function(xhr) {
  xhr.setRequestHeader("X-Device-Token", AppInfo.getToken());
}

// private
RequestsFactory.prototype.getRequestError = function(xhr, textStatus, errorThrown) {
  var error = RequestErrors.UNKNOWN;
  if ('timeout' == textStatus) {
    error = RequestErrors.TIMEOUT;
  } else if (500 == xhr.status) {
    error = RequestErrors.INTERNAL_SERVER_ERROR;
  } else if (503 == xhr.status) {
    error = RequestErrors.SERVICE_UNAVAILABLE;
  }

  return error;
}

